/*
  Web client

 This sketch connects to a website (http://www.google.com)
 using an Arduino Wiznet Ethernet shield.

 Circuit:
 * Ethernet shield attached to pins 10, 11, 12, 13
 * pin 4 is also used
 * usable pins: 2,3,5,6,7,8,9,14,15,16,17

 created 18 Dec 2009
 by David A. Mellis
 modified 9 Apr 2012
 by Tom Igoe, based on work by Adrian McEwen

ADD LCD?
add control switches for all channels?

 */

#include <OneWire.h>
#include <SPI.h>
#include <Ethernet.h>
#include <LiquidCrystal.h>

OneWire ds(2); // on digital pin 3
//LiquidCrystal lcd(9, 8, 7, 6, 3, 2);
LiquidCrystal lcd(3, 5, 6, 7, 8, 9); //finished product pinout
// backlight pin 14

// Enter a MAC address for your controller below.
// Newer Ethernet shields have a MAC address printed on a sticker on the shield
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xCA, 0xFE };
// if you don't want to use DNS (and reduce your sketch size)
// use the numeric IP instead of the name for the server:
IPAddress server(192,168,1,30);  // numeric IP for Google (no DNS)
//char server[] = "your.webserver.hostname.here";    // name address for Google (using DNS)

// Set the static IP address to use if the DHCP fails to assign
IPAddress ip(192, 168, 1, 33);

// Initialize the Ethernet client library
// with the IP address and port of the server
// that you want to connect to (port 80 is default for HTTP):
EthernetClient client;

// Generally, you should use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long previousMillis = 0;        // will store last time LED was updated
unsigned long previousMillis1 = 0;        // temp sensor read interval.

// constants won't change :
long interval = 300000;           // interval at which to send temperature (milliseconds) 600K = 10 minutes
long interval1 = 1000;          // temp sensor read interval

String getline;

float temparray[20];
int arraypointer=0;

float temperature;
float avgtemp;
int temp;
int deci;
int tempc;

byte cc0[8] = {
  0b00110,
  0b01001,
  0b01001,
  0b00110,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

byte cc1[8] = {
  0b00011,
  0b00111,
  0b01111,
  0b01111,
  0b01111,
  0b01111,
  0b00111,
  0b00011
};

byte cc2[8] = {
  0b00000,
  0b00000,
  0b11111,
  0b00000,
  0b00000,
  0b11111,
  0b00000,
  0b00000
};

byte cc3[8] = {
  0b00000,
  0b00000,
  0b11111,
  0b11100,
  0b11100,
  0b11111,
  0b00000,
  0b00000
};

byte cc4[8] = {
  0b00000,
  0b00000,
  0b11110,
  0b00001,
  0b00001,
  0b11110,
  0b00000,
  0b00000
};

byte cc5[8] = {
  0b11000,
  0b11100,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b11100,
  0b11000
};

byte cc6[8] = {
  0b00000,
  0b00000,
  0b11111,
  0b11111,
  0b11111,
  0b11111,
  0b00000,
  0b00000
};

byte cc7[8] = {
  0b00000,
  0b00000,
  0b11110,
  0b11111,
  0b11111,
  0b11110,
  0b00000,
  0b00000
};

void setup() {
  pinMode(14, OUTPUT);
  digitalWrite(14, HIGH); //LCD BACKLIGHT?
  
  lcd.createChar(0, cc0);
  lcd.createChar(1, cc1);
  lcd.createChar(2, cc2);
  lcd.createChar(3, cc3);
  lcd.createChar(4, cc4);
  lcd.createChar(5, cc5);
  lcd.createChar(6, cc6);
  lcd.createChar(7, cc7);

  // Open serial communications and wait for port to open:
  Serial.begin(9600);
  //while (!Serial) {
  //  ; // wait for serial port to connect. Needed for native USB port only
  //}

  // start the Ethernet connection:
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    // try to congifure using IP address instead of DHCP:
    Ethernet.begin(mac, ip);
  }
  // give the Ethernet shield a second to initialize:
  delay(1000);
  
  lcd.begin(16, 2);
  lcd.print(Ethernet.localIP());
  delay(2000);
  digitalWrite(14, LOW); //LCD BACKLIGHT?
}

void loop() {

  unsigned long currentMillis1 = millis();

  if (currentMillis1 - previousMillis1 >= interval1) {
    // save the last time you blinked the LED
    previousMillis1 = currentMillis1;
    temperature = getTemp();

    if (temparray[0]==0) { //happens only after boot
      for (int x = 0; x < 20; x++) {
        temparray[x]=temperature;
      }
    }
    //load into array
    temparray[arraypointer]=temperature;
    arraypointer++;
    if (arraypointer>19) {arraypointer=0;}
    
    temp=int(temperature);
    deci=int((temperature-(float)temp)*100);
    lcd.clear();
    lcd.setCursor(0, 0);
    if (deci<10) {
      Serial.println("Temp read: " + (String)temp + ",0" + (String)deci);  //debug
      lcd.print("         " + (String)temp + ",0" + (String)deci + char(223) + "C");
    } else {
      Serial.println("Temp read: " + (String)temp + "," + (String)deci);  //debug
      lcd.print("         " + (String)temp + "," + (String)deci + char(223) + "C");
    }
    tempc=temp;
    avgtemp=0;
    for (int x = 0; x < 20; x++) {
      avgtemp=avgtemp+temparray[x];
    }
    avgtemp=avgtemp/20;    
    temp=int(avgtemp);
    deci=int((avgtemp-(float)temp)*100);
    
    lcd.setCursor(0, 1);
    if (deci<10) {
      Serial.println("Avgr read: " + (String)temp + ",0" + (String)deci);  //debug
      lcd.print("Average: " + (String)temp + ",0" + (String)deci + char(223) + "C");
    } else {
      Serial.println("Avgr read: " + (String)temp + "," + (String)deci);  //debug
      lcd.print("Average: " + (String)temp + "," + (String)deci + char(223) + "C");
    }
    
//    replace with custom char
      lcd.setCursor(14,0);
      lcd.write((uint8_t)0);
      lcd.setCursor(14,1);
      lcd.write((uint8_t)0);

      lcd.setCursor(0,0);
      lcd.write((uint8_t)1);
      lcd.write((uint8_t)5);
      lcd.write((uint8_t)2);
      lcd.write((uint8_t)2);
      lcd.write((uint8_t)2);
      lcd.write((uint8_t)2);
      lcd.write((uint8_t)2);
      lcd.write((uint8_t)4);
      
      //lcd.setCursor((constrain(tempc,20,24)-20)+2,0);
      //lcd.write((uint8_t)3);

      for (int i=2; i <= (constrain(tempc,20,23)-20)+2; i++){
        lcd.setCursor(i,0);
        lcd.write((uint8_t)6);
      } 

      lcd.setCursor((constrain(tempc,20,24)-20)+2,0);
      lcd.write((uint8_t)3);

      if (tempc>=25) {
        lcd.setCursor(6,0);
        lcd.write((uint8_t)6);
      }
      if (tempc>=26) {
        lcd.setCursor(7,0);
        lcd.write((uint8_t)7);
      }
      if (tempc<=19) {
        lcd.setCursor(2,0);
        lcd.write((uint8_t)2);
      }
      
      
    interval1 = 10000;
  }


    unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    // save the last time you blinked the LED
    previousMillis = currentMillis;

    
    Serial.println("connecting...");
  // if you get a connection, report back via serial:
  if (client.connect(server, 80)) {
    Serial.println("connected");
    // Make a HTTP request:
    if (deci<10) {
      getline=String("GET /write.asp?value=" + (String)temp + ",0" + (String)deci + "&id=003 HTTP/1.1");
    } else {
      getline=String("GET /write.asp?value=" + (String)temp + "," + (String)deci + "&id=003 HTTP/1.1");
    }
    
    
    //client.println("GET /write.asp?value=" + (int)temperature + "&id=test HTTP/1.1");  //
    client.println(getline);  //
    client.println("Host: your.server.here");  
    client.println("Connection: close");
    client.println();
    interval = 300000;
  } else {
    // if you didn't get a connection to the server:
    Serial.println("connection failed");
    client.stop();
    //red LED?
  }
  
  // if there are incoming bytes available
  // from the server, read them and print them:
  if (client.available()) {
    char c = client.read();
    Serial.print(c);
  }

  // if the server's disconnected, stop the client:
  if (!client.connected()) {
    Serial.println();
    Serial.println("disconnecting.");
    client.stop();

    // do nothing forevermore:
  //  while (true);
  }
  client.stop();

  lcd.createChar(0, cc0);
  lcd.createChar(1, cc1);
  lcd.createChar(2, cc2);
  lcd.createChar(3, cc3);
  lcd.createChar(4, cc4);
  lcd.createChar(5, cc5);
  lcd.createChar(6, cc6);
  lcd.createChar(7, cc7);
  lcd.noCursor();
 }
}

float getTemp(){
//returns the temperature from one DS18S20 in DEG Celsius

byte data[12];
byte addr[8];


// error checking
if ( !ds.search(addr)) {
  //no more sensors on chain, reset search
  ds.reset_search();
  return -1000;
}

if ( OneWire::crc8( addr, 7) != addr[7]) {
  //Serial.println("CRC is not valid!");
  return -1000;
}

if ( addr[0] != 0x10 && addr[0] != 0x28) {
  //Serial.print("Device is not recognized");
  return -1000;
}


//read DS
ds.reset();
ds.select(addr);
ds.write(0x44,1); // start conversion, with parasite power on at the end

delay(1000); //give power to the DS18B20 to perform convert.

byte present = ds.reset();
ds.select(addr);
ds.write(0xBE); // Read Scratchpad


for (int i = 0; i < 9; i++) { // we need 9 bytes
data[i] = ds.read();
}

ds.reset_search();

byte MSB = data[1];
byte LSB = data[0];

float tempRead = ((MSB << 8) | LSB); //using two's compliment
float TemperatureSum = tempRead / 16;

return TemperatureSum;

}
